#/////////////////////////////////////////////////////////////////
#//	 (c) 2012 by mister-man										//
#//	  License:(CC BY-NC-ND 3.0)									//
#//   http://creativecommons.org/licenses/by-nc-nd/3.0/de/		//
#/////////////////////////////////////////////////////////////////

import es
import pymysql
import time
import cfglib
from os.path import join, isdir
from os import mkdir

info = es.AddonInfo()
info.author      = "mister-man"
info.name        = "smartDB"
info.version     = "0.8.3"
info.url         = "http://c-o-g.de"
info.description = "Database tool for CSS + Python"
info.basename	 = "smartDB"
	
es.ServerVar(info.name + "_version", info.version).makepublic()
es.ServerVar(info.name + "_author", info.author).makepublic()
es.ServerVar(info.name + "_url", info.url).makepublic()

config = cfglib.AddonCFG(es.getAddonPath(info.basename) + '/cfg/smartDB.cfg')
config.text("smartDB settings:")
config.text("")
config.text("MySQL settings:")
mysqlHost = config.cvar("mysqlHost", "localhost", "Host adress of the My Sql server (default: localhost)")
mysqlPort = int(config.cvar("mysqlPort", "3306", "Port of the My Sql server (default: 3306)"))
mysqlUser = config.cvar("mysqlUser", "smartDB", "Username for My Sql login (default: smartDB)")
mysqlPasswd = config.cvar("mysqlPasswd", "smartDB", "Password for My Sql login (default: smartDB)")
mysqlDatabase = config.cvar("mysqlDatabase", "smartDB", "Database name on My Sql serevr (default: smartDB)")
mysqlPraefix = config.cvar("mysqlPraefix", "", "Database prefix")
config.text("")
config.text("Server settings:")
edb_keycmd = int(config.cvar("edb_keycmd", "0", "Override es.key commands with edb.key commands"))
config.write()
config.execute()


####################
#	Load / Unload  #
####################

def load():
	pass
	
def unload():
	edb.disconnect()
	

####################
#	Funktionen     #
####################

class logging(object):
	def __init__(self, LOGLEVEL=0):
		self.set_level(LOGLEVEL)
		
	def set_level(self, LOGLEVEL):
		self.__loglevel = LOGLEVEL
		
	def __log_silent__(self , message):
		path = join(es.getAddonPath('smartDB'), 'logs')
		if not isdir(path):
			mkdir(path)
		path = join(path, time.strftime('%Y%m%d.log'))
		file = open(path, 'a+')
		file.write(time.strftime('L %m/%d/%Y - %H:%M:%S: ')+str(message)+'\n')
		file.close()
		
	def __log__(self ,  message):
		es.dbgmsg(0, str(message))
		
	def __log_client__(self , message):
		es.msg('#default' , message)
		
	def log(self , message):
		if self.__loglevel > 0:
			message = '[smartDB] %s' %(message)
			if self.__loglevel == 2:
				# silent logging 
				self.__log_silent__(message)
			elif self.__loglevel == 4:
				# logging
				self.__log__(message)
			elif self.__loglevel == 7:
				# log client
				self.__log_client__(message)
			elif self.__loglevel == 6:
				# silent + logging
				self.__log_silent__(message)
				self.__log__(message)
			elif self.__loglevel == 9:
				# log + client
				self.__log_silent__(message)
				self.__log_client__(message)
			elif self.__loglevel == 11:
				# log + client
				self.__log__(message)
				self.__log_client__(message)
			elif self.__loglevel == 13:
				self.__log_silent__(message)
				self.__log__(message)
				self.__log_client__(message)
			else:
				#log fail
				message = '[smartDB]: Invalid loglevel %s' %(self.__loglevel)
				self.__log_silent__(message)
				self.__log__(message)
				self.__log_client__(message)
				
	def consolemsg(self, message):
		message = '[smartDB] %s' %(message)
		self.__log__(message)
				
log = logging(6)

class mysql(object): 
	def __init__(self , HOST , PORT , USER , PASSWD , DATABASE, RETRY=3):
		self.__host = HOST
		self.__port = PORT
		self.__user = USER
		self.__passwd = PASSWD
		self.__database = DATABASE
		self.__max_retry = RETRY
		self.__retry = RETRY
		self.__db = self.___connect___()
		self.cursor = self.__set_cursor__()
		
	def ___connect___(self):
		log.log('Connect to My SQL Server')
		self.__retry -= 1
		try:
			return pymysql.connect(host=str(self.__host), port=int(self.__port), user=str(self.__user), passwd=str(self.__passwd), db=str(self.__database))
		except:
			if self.__retry >= 0:
				log.log('Can not connect to MySQL Server, retry')
				self.___connect___()
			else:
				log.log('Can not connect to MySQL Server')
				
	def __checking___(self):
		try:
			self.__db.ping()
		except:
			log.log('Connection to MySQL Server lost, retry')
			self.__db = self.___connect___()
			self.cursor = self.__set_cursor__()
			
	def ___del_connection___(self):
		try:
			self.__db.close()
			log.log('Connection Closed')
		except:
			log.log('Can not close connect to MySQL Server, not connected')
			
	def __set_cursor__(self):
		try:
			return self.__db.cursor()
		except:
			log.log('Can not set cursor')
		
	def __del_cursor__(self):
		try:
			self.cursor.close()
		except:
			log.log('Can not close cursor')
			
	def disconnect(self):
		self.__del_cursor__()
		self.___del_connection___()
			
		
class easyDB(object): 
	def __init__(self , HOST , PORT , USER , PASSWD , DATABASE):
		self.__database = DATABASE
		self.__sql = mysql(HOST , PORT , USER , PASSWD , self.__database)
		self.__groups = self.___list_table___()
		
	def ___list_table___(self):
		try:
			mysql = "SHOW TABLES FROM %s;" %(self.__database)
			self.__sql.cursor.execute(str(mysql))
			return [row[0] for row in self.__sql.cursor]
		except:
			log.log('SQL failure by getting grouplist from %s' %(self.__database))
			
	def __list_keys__(self , key):
		try:
			pass
		except:
			log.log('SQL failure by getting keylist %s from group %s' %(key , self.__database))
			
	def disconnect(self):
		self.__sql.disconnect()
		
	def grouplist(self , *args):
		# grouplist() or grouplist(<groupname>)
		self.__groups = self.___list_table___()
		if len(args) == 0:
			msg = 'Grouplist:'
			for groupname in self.__groups:
				msg +=  ('\n\t\t- ' + groupname)
			log.consolemsg(str(msg))
		elif len(args) == 1:
			if args[0] in self.__groups:
				try:
					mysql = "SELECT columnkey,linekey,value FROM %s ORDER BY columnkey;" %(args[0])
					self.__sql.cursor.execute(str(mysql))
					rows = self.__sql.cursor.fetchall()
					msg = 'Group: %s' %(args[0])
					last_key ='none'
					for row in rows:
						if row[0] != last_key:
							last_key = row[0]
							msg +=  ('\n\t' + '"' + row[0] + '"')
							msg +=  ('\n\t\t' + '"' + row[1] + '"' + '\t\t' + '"' + row[2] + '"')
						else:
							msg +=  ('\n\t\t' + '"' + row[1] + '"' + '\t\t' + '"' + row[2] + '"')
					log.consolemsg(str(msg))
				except:
					log.log('SQL failure by listing group %s' %(args[0]))
			else:
				log.log('Unkown group %s' %(args[0]))
		else:
			log.log('Syntax error grouplist() or grouplist(<groupname>)')
				
	def groupcreate(self , *args):
		# groupcreate('groupname')
		if len(args) == 1:
			try:
				mysql = "CREATE TABLE IF NOT EXISTS %s (\
							columnkey VARCHAR(45) NOT NULL ,\
							linekey VARCHAR(45) NOT NULL ,\
							value VARCHAR(100) NOT NULL ,\
							PRIMARY KEY (columnkey, linekey)\
						)" %(args[0])
				self.__sql.cursor.execute(str(mysql))
			except:
				log.log('SQL failure by creating group %s' %(args[0]))
		else:
			log.log('Syntax error groupcreate(<groupname>)')
			
	def keycreate(self , *args):
		# keycreate(<groupname>,<keyname>)
		# dummy funktion
		pass
		
	def groupsave(self , *args):
		# groupsave(<groupname>,<location>)
		# dummy funktion
		pass
		
	def groupload(self , *args):
		# keycreate(<groupname>,<location>)
		# dummy funktion
		pass
		
	def keysetvalue(self , *args):
		# keysetvalue(<groupname>,<keyname>,<valuename>,<value>)
		# return
		self.__groups = self.___list_table___()
		if len(args) == 4:
			if args[0] in self.__groups:
				try:
					mysql = "SELECT columnkey,linekey,value FROM %s WHERE columnkey='%s' AND linekey='%s';" %(args[0] , args[1] , args[2])
					self.__sql.cursor.execute(str(mysql))
					rowcount = self.__sql.cursor.rowcount
					if rowcount == 0:
						mysql = "INSERT INTO %s (`columnkey`, `linekey`, `value`) VALUES ('%s', '%s', '%s');" %(args[0] , args[1] , args[2] , args[3])
						self.__sql.cursor.execute(str(mysql))
					else:
						row = self.__sql.cursor.fetchone()
						mysql = "UPDATE %s SET columnkey = '%s', linekey = '%s', value = '%s' WHERE columnkey = '%s' AND `linekey` = '%s';" %(args[0] , args[1] , args[2] , args[3] , row[0] , row[1])
						self.__sql.cursor.execute(str(mysql))
				except:
					log.log('SQL failure by creating keyvalue %s %s %s' %(args[0] ,  args[1] , args[2]))
			else:
				log.log('Unkown group %s' %(args[0]))
		else:
			log.log('Syntax error keysetvalue(<groupname>,<keyname>,<valuename>)')
			
	def keygetvalue(self , *args):
		# keygetvalue(<groupname>,<keyname>,<valuename>)
		self.__groups = self.___list_table___()
		if len(args) == 3:
			if args[0] in self.__groups:
				try:
					mysql = "SELECT value FROM %s WHERE columnkey='%s' AND linekey='%s';" %(args[0] , args[1] , args[2])
					self.__sql.cursor.execute(str(mysql))
					row = self.__sql.cursor.fetchone()
					return row[0]
				except:
					log.log('SQL failure by getting keyvalue %s %s %s' %(args[0] ,  args[1] , args[2]))
			else:
				log.log('Unkown group %s' %(args[0]))
		else:
			log.log('Syntax error keygetvalue(<groupname>,<keyname>,<valuename>)')
			
	def groupdelete(self , *args):
		# groupdelete(<keygroupname>)
		self.__groups = self.___list_table___()
		if len(args) == 1:
			if args[0] in self.__groups:
				try:
					mysql = "DROP TABLE %s" %(args[0])
					self.__sql.cursor.execute(str(mysql))
				except:
					log.log('SQL failure by deleting group %s' %(args[0]))
			else:
				log.log('Unkown group %s' %(args[0]))
		else:
			log.log('Syntax error groupdelete(<keygroupname>)')

	def groupname(self , *args):
		# groupname(<keygroupname> , <new keygroupname>)
		self.__groups = self.___list_table___()
		if len(args) == 1:
			if args[0] in self.__groups:
				try:
					mysql = "RENAME TABLE %s TO %s;" %(args[0] , args[1])
					self.__sql.cursor.execute(str(mysql))
				except:
					log.log('SQL failure by renaming group %s in %s' %(args[0] , args[1]))
			else:
				log.log('Unkown group %s' %(args[0]))
		else:
			log.log('Syntax error groupname(<keygroupname> , <new keygroupname>)')
			
	def keydelete(self , *args):
		# keydelete(<keygroupname> ,<keyname>)
		self.__groups = self.___list_table___()
		if len(args) == 2:
			if args[0] in self.__groups:
				try:
					mysql = "DELETE FROM %s WHERE columnkey = '%s'" %(args[0] , args[1])
					self.__sql.cursor.execute(str(mysql))
				except:
					log.log('SQL failure by deleting key %s in group %s' %(args[1] , args[0]))
			else:
				log.log('Unkown group %s' %(args[0]))
		else:
			log.log('Syntax error keydelete(<keygroupname> ,<keyname>)')

edb = easyDB(mysqlHost , mysqlPort , mysqlUser , mysqlPasswd , mysqlDatabase)
			
if edb_keycmd == 1:
	es.keycreate = edb.keycreate
	es.keydelete = edb.keydelete
	es.keygetvalue = edb.keygetvalue
	es.keysetvalue = edb.keysetvalue
	es.keygroupcreate = edb.groupcreate
	es.keygroupload = edb.groupload
	es.keygrouprename = edb.groupname
	es.keygroupsave = edb.groupsave
	es.keylist = edb.grouplist


####################
#	  Events       #
####################

def server_shutdown(ev):
	edb.disconnect()